require 'test_helper'

class Infrastructure::ConfigerableTest < MiniTest::Test

  def setup
    @class = Class.new do
      include Infrastructure::Configurable
      attr_reader :before_configure_callback
      attr_reader :after_configure_callback
      attr_reader :before_assignment_callback
      attr_reader :after_assignment_callback
      def initialize
        callback :before_configure, -> { @before_configure_callback = true }
        callback :after_configure, -> { @after_configure_callback = true }
        callback :before_assignment, -> (key, value) { @before_assignment_callback = true }
        callback :after_assignment, -> (key, value) { @after_assignment_callback = true }
      end
    end
    @instance = @class.new
  end

  def test_configure
    @instance.configure do
      config.key1 = 'value1'
      assign :key2, 'value2'
      config.key3 = {
        key4: 'value4',
        key5: { key6: 'value6' }
      }
    end
    assert_instance_of Infrastructure::Configuration, @instance.configuration
    assert_equal 'value1', @instance.key1, 'config.key ='
    assert_equal 'value2', @instance.configuration.key2, 'assign key, value'
    @instance.configuration.key2 = 'value2.1'
    assert_equal 'value2.1', @instance.configuration.key2, 'configuration.key ='
    @instance.configuration.assign :key2, 'value2.2'
    assert_equal 'value2.2', @instance.configuration.key2, 'configuration.assign key, value'
    assert_equal 'value6', @instance.key3.key5.key6, 'configuration.assign nested hash'
    assert_equal @instance.configuration.fetch(:key3), @instance.configuration.key3, 'configuration.assign key, value == configuration.fetch(key)'
    assert_raises NoMethodError do
      @instance.blah = 'test'
    end
    assert_raises NoMethodError do
      @instance.blah
    end
  end

  def test_callbacks
    refute @instance.before_configure_callback, 'before_configure callback'
    refute @instance.after_configure_callback, 'after_configure callback'
    refute @instance.before_assignment_callback, 'before_assignment callback'
    refute @instance.after_assignment_callback, 'after_assignment callback'
    @instance.configure {}
    assert @instance.before_configure_callback, 'before_configure callback'
    assert @instance.after_configure_callback, 'after_configure callback'
    refute @instance.before_assignment_callback, 'before_assignment callback'
    refute @instance.after_assignment_callback, 'after_assignment callback'
    @instance.configuration.assign :key, :value
    assert @instance.before_assignment_callback, 'before_assignment callback'
    assert @instance.after_assignment_callback, 'after_assignment callback'
  end

  def test_load_config
    @instance.configuration.load_config File.expand_path('../configuration_test.yml.erb', __FILE__) do
      @success = true
    end
    assert_equal 'value1', @instance.key1, 'config file should ready simple key/value pairs'
    assert_equal 'value3', @instance.key2.key3, 'hashes should be convered to configuration instances'
    assert_equal 'value5', @instance.configuration.key2.key4.key5, 'hashes should be convered to configuration instances'
    assert @success, 'load_config block should be run if file loaded'
    assert_raises Errno::ENOENT do
      @instance.configuration.load_config File.expand_path('../DNE', __FILE__) do
        @failure = true
      end
    end
    @instance.configuration.load_config_if_exists File.expand_path('../DNE', __FILE__) do
      @failure = true
    end
    assert @failure.nil?, 'load_config block should not be run if config not found'
  end

  def test_save_config
    @instance.configuration.load_config File.expand_path('../configuration_test.yml.erb', __FILE__)
    @instance.configuration.save_config File.expand_path('../configuration_save_test.yml.erb', __FILE__)
    @other = Class.new do
      include Infrastructure::Configurable
    end.new
    @other.configuration.load_config File.expand_path('../configuration_save_test.yml.erb', __FILE__)
    assert @other.configuration.to_h == @instance.configuration.to_h, 'configuration should save and reload the same'
    File.delete File.expand_path('../configuration_save_test.yml.erb', __FILE__)
  end


end
